@@ -107,7 +107,7 @@ class CloudvectorDAST(object):
                 self.static_headers_from_cfg = self.cv_config.get("static_headers")
                 if self.cv_config.get("authentication"):
                     self.cvdast_to_auth_info = self.cv_config.get("authentication")
-                self.events_to_ignore = self.cv_config.get("headers_to_ignore_cv_events")
+                self.events_to_ignore = self.cv_config.get("headers_to_ignore_cv_events",{})
         if str(self.cv_config.get("generate_tests_for_response_codes", "Y")).lower() == "y":
             self.generate_tests_for_respcodes = True
         else:
@@ -324,7 +324,7 @@ class CloudvectorDAST(object):
         #with open("tests/data/params_info_from_spec.json", "w+") as _file:
         with open(os.path.join(os.path.join("tests","data"),"params_info_from_spec.json"), "w+") as _file:
             json.dump(params_info_from_spec, _file)
-        print(params_info_from_spec,"<<<<<<")
+
         self.api_with_array_payload = list(set(self.api_with_array_payload))
         return apis_from_spec
 
@@ -437,15 +437,18 @@ class CloudvectorDAST(object):
         if isinstance(self.params_details_from_spec, dict):
             for param, info in self.params_details_from_spec.items():
                 _params = []
-                if type(info) is dict:
-                    for k, v in keypaths(info):
-                        more_params.append(k[-1])
-                        _params.append(k[-1])
-                elif type(info) is list:
-                    for each in info:
-                        for k, v in keypaths(each):
+                try:
+                    if type(info) is dict:
+                        for k, v in keypaths(info):
                             more_params.append(k[-1])
                             _params.append(k[-1])
+                    elif type(info) is list:
+                        for each in info:
+                            for k, v in keypaths(each):
+                                more_params.append(k[-1])
+                                _params.append(k[-1])
+                except:
+                    pass
                 nested_params[param] = _params
                 more_params = list(set(more_params))
 
@@ -605,6 +608,7 @@ class CloudvectorDAST(object):
 
     def _process_cv_events(self, cv_events):
         apis_to_be_tested = {}
+        host_url = ""
 
         try:
             new_spec_info = self._scan_input_spec(self.apispec_two_path)
@@ -632,6 +636,7 @@ class CloudvectorDAST(object):
                 if api not in apis_to_be_tested:
                     apis_to_be_tested[api] = {}
                 method = event["method"].lower()
+
                 if method not in apis_to_be_tested[api]:
                     apis_to_be_tested[api][method] = {}
                 if self.static_headers_from_cfg:
@@ -644,12 +649,18 @@ class CloudvectorDAST(object):
                 apis_to_be_tested[api][method]["url"] = event["url"].replace(host_url,
                                                                              "")  # str(event["host"]).lower() + event["url"]
                 apis_to_be_tested[api][method]["url_from_spec"] = event["api_queried"]
+
                 if apis_to_be_tested[api][method].get("params"):
                     known_params = apis_to_be_tested[api][method].get("params")
                 else:
                     known_params = []
-                apis_to_be_tested[api][method]["params"] = new_spec_info.get(
-                    str(event["http-req-url"]).replace("//", "/"), {}).get(method, [])
+
+                if apis_to_be_tested[api][method].get("params"):
+                    apis_to_be_tested[api][method]["params"] += new_spec_info.get(
+                        str(event["http-req-url"]).replace("//", "/"), {}).get(method, [])
+                else:
+                    apis_to_be_tested[api][method]["params"] = new_spec_info.get(
+                        str(event["http-req-url"]).replace("//", "/"), {}).get(method, [])
 
                 params_in_traffic = event.get("body", {}).keys()
 
@@ -664,6 +675,7 @@ class CloudvectorDAST(object):
                 apis_to_be_tested[api][method]["params"].append("host")
                 apis_to_be_tested[api][method]["params"].append("url_prefix")
                 apis_to_be_tested[api][method]["params"].append("access_token")
+                apis_to_be_tested[api][method]["params"] = list(set(apis_to_be_tested[api][method]["params"]))
 
                 self._create_assertions_map(event["http-req-url"].replace(host_url, ""),
                                             apis_to_be_tested[api][method]["params"], event["body"],
@@ -680,7 +692,7 @@ class CloudvectorDAST(object):
             template = Template(file_.read())
 
         host_url, apis_to_be_tested = self._process_cv_events(cv_events)
-        print(host_url, apis_to_be_tested)
+
         for k, v in apis_to_be_tested.items():
             # code = template.render(api_info=apis_to_be_tested[k], api_name=k, host_url=host_url)
             filename = str(k).replace("/", "_").replace("{", "").replace("}", "")
@@ -698,7 +710,6 @@ class CloudvectorDAST(object):
                 status_codes_for_api = self.respcodes_from_spec.get(api_name, {})
             else:
                 status_codes_for_api = {}
-
             with open("tests/test_" + str(filename) + ".py", 'w+') as fh:
                 fh.write(template.render(apis_metadata=apis_to_be_tested[k], api_name=k, host_url=host_url,
                                          custom_validations=extra_validations, STATUS_CODES=status_codes_for_api,
@@ -1210,6 +1221,9 @@ class CloudvectorDAST(object):
         for collection in params_collection:
             for k, v in collection.items():
                 if k.startswith("$") or k=="header":
+                    if "header" in str(k).lower():
+                        input_params[k] = v
+                        #input_params["$headers"]= list(v.keys())
                     #print("skipping1", k, v)
                     continue
                 if k not in input_params:
@@ -1261,6 +1275,9 @@ class CloudvectorDAST(object):
         for collection in params_collection:
             for k, v in collection.items():
                 if k.startswith("$") or "date" in str(k).lower() or "time" in str(k).lower() or "header" in str(k).lower():
+                    if "header" in str(k).lower():
+                        input_params[k] = v
+                        #input_params["$headers"] = list(v.keys())
                     continue
                 if k not in input_params:
                     input_params[k] = {}
@@ -1357,6 +1374,7 @@ class CloudvectorDAST(object):
                 random_data_set = random.choice(data_set)
                 start_datetime_key = ""
                 end_datetime_key = ""
+
                 for k, v in random_data_set.items():
                     if "starttime" in str(k).lower() or "startdate" in str(k).lower():
                         start_datetime_key = k
@@ -1364,12 +1382,13 @@ class CloudvectorDAST(object):
                         end_datetime_key = k
                 header = random_data_set.get("header")
                 if header:
-                    del random_data_set["header"]
+                   del random_data_set["header"]
                 #     random_data_set.update(header)
+
                 for k, v in random_data_set.items():
-                    if k.startswith("$"):
+                    if k.startswith("$") or "authorization" in str(k).lower():
                         continue
-                    if k in method_info.get("required"):
+                    if str(k).lower() in [str(_).lower() for _ in method_info.get("required")]:
                         error_data_block = deepcopy(random_data_set)
                         if k == start_datetime_key and end_datetime_key:
                             error_data_block[k] = error_data_block[end_datetime_key]
@@ -1506,12 +1525,14 @@ class CloudvectorDAST(object):
             for method, minfo in info.items():
                 print(api+" --> "+method)
                 for code, dset in minfo.items():
+                    print(code, dset)
                     dset = list(set(dset))
                     print("\t\t\t\tResponse Code to be triggered: "+str(code))
                     print("\t\t\t\t# of data sets(test cases): "+str(len(dset)))
                     print("\t\t\t\tReason(s) for the Test data: " + str(dset))
                     total_tests+=len(dset)
             print("-------"*15)
+        print("\t\t\t\tTotal test case(s) generated: " + str(total_tests))
 
         if os.path.exists("tests"):
             with open("tests/input_params_auto-generated.json", "w+") as fobj:
@@ -1578,6 +1599,8 @@ def main():
                 properties_file_path = sys.argv[prop_file_index]
             except ValueError:
                 print("DEBUG: properties file not passed!")
+        if os.path.exists("properties.yaml"):
+            os.remove("properties.yaml")
         if properties_file_path:
             if os.path.exists(properties_file_path):
                 try:
